@@ -1,19 +1,21 @@
 import torch
 from torch.optim import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, LOSS, PARAMETERS
 
 
 class LARS(Optimizer, BaseOptimizer):
-    r"""Layer-wise Adaptive Rate Scaling (no rate scaling or weight decay for parameters <= 1D)
+    r"""Layer-wise Adaptive Rate Scaling (no rate scaling or weight decay for parameters <= 1D).
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
     :param weight_decay: float. weight decay (L2 penalty).
     :param momentum: float. momentum.
+    :param dampening: float. dampening for momentum.
     :param trust_coefficient: float. trust_coefficient.
+    :param nesterov: bool. enables nesterov momentum.
     :param eps: float. epsilon.
     """
 
@@ -23,23 +25,29 @@ class LARS(Optimizer, BaseOptimizer):
         lr: float = 1e-3,
         weight_decay: float = 0.0,
         momentum: float = 0.9,
-        trust_coefficient: float = 0.001,
+        dampening: float = 0.0,
+        trust_coefficient: float = 1e-3,
+        nesterov: bool = False,
         eps: float = 1e-6,
     ):
         self.lr = lr
         self.weight_decay = weight_decay
         self.momentum = momentum
+        self.dampening = dampening
         self.trust_coefficient = trust_coefficient
+        self.nesterov = nesterov
         self.eps = eps
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            weight_decay=weight_decay,
-            momentum=momentum,
-            trust_coefficient=trust_coefficient,
-        )
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'weight_decay': weight_decay,
+            'momentum': momentum,
+            'dampening': dampening,
+            'trust_coefficient': trust_coefficient,
+            'nesterov': nesterov,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -50,7 +58,7 @@ class LARS(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'Lars'
 
     @torch.no_grad()
@@ -68,35 +76,42 @@ class LARS(Optimizer, BaseOptimizer):
             with torch.enable_grad():
                 loss = closure()
 
-        for g in self.param_groups:
-            for p in g['params']:
+        for group in self.param_groups:
+            for p in group['params']:
                 if p.grad is None:
                     continue
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
+                    raise NoSparseGradientError(self.__str__)
 
                 if p.ndim > 1:  # if not normalization gamma/beta or bias
-                    grad = grad.add(p, alpha=g['weight_decay'])
-                    param_norm = torch.norm(p)
-                    update_norm = torch.norm(grad)
+                    param_norm = p.norm(2.0)
+                    update_norm = grad.norm(2.0)
                     one = torch.ones_like(param_norm, device=param_norm.device)
 
-                    q = torch.where(
+                    trust_ratio = torch.where(
                         param_norm > 0.0,
-                        torch.where(update_norm > 0.0, (g['trust_coefficient'] * param_norm / update_norm), one),
+                        torch.where(update_norm > 0.0, (group['trust_coefficient'] * param_norm / update_norm), one),
                         one,
                     )
-                    grad = grad.mul(q)
 
-                param_state = self.state[p]
-                if 'mu' not in param_state:
-                    param_state['mu'] = torch.zeros_like(p, device=p.device)
+                    grad.add_(p, alpha=group['weight_decay'])
+                    grad.mul_(trust_ratio)
 
-                mu = param_state['mu']
-                mu.mul_(g['momentum']).add_(grad)
+                if group['momentum'] > 0.0:
+                    param_state = self.state[p]
+                    if 'momentum_buffer' not in param_state:
+                        param_state['momentum_buffer'] = grad.clone().detach()
 
-                p.add_(mu, alpha=-g['lr'])
+                    mb = param_state['momentum_buffer']
+                    mb.mul_(group['momentum']).add_(grad, alpha=1.0 - group['dampening'])
+
+                    if group['nesterov']:
+                        grad.add_(mb, alpha=group['momentum'])
+                    else:
+                        grad.copy_(mb)
+
+                p.add_(grad, alpha=-group['lr'])
 
         return loss
