@@ -1,14 +1,14 @@
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, LOSS, PARAMETERS
 from pytorch_optimizer.optimizer.utils import neuron_mean, neuron_norm
 
 
 class Nero(Optimizer, BaseOptimizer):
-    """Learning by Turning: Neural Architecture Aware Optimisation
+    """Learning by Turning: Neural Architecture Aware Optimisation.
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -22,7 +22,7 @@ class Nero(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(lr=lr, constraints=constraints)
+        defaults: DEFAULTS = {'lr': lr, 'constraints': constraints}
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -30,7 +30,7 @@ class Nero(Optimizer, BaseOptimizer):
         self.validate_beta(self.beta)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'Nero'
 
     @torch.no_grad()
@@ -64,7 +64,7 @@ class Nero(Optimizer, BaseOptimizer):
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
+                    raise NoSparseGradientError(self.__str__)
 
                 state = self.state[p]
                 if len(state) == 0:
