@@ -3,14 +3,14 @@ import math
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import BETAS, CLOSURE, DEFAULTS, LOSS, PARAMETERS
 from pytorch_optimizer.optimizer.gc import centralize_gradient
 
 
 class Ranger(Optimizer, BaseOptimizer):
-    r"""a synergistic optimizer combining RAdam and LookAhead, and now GC in one optimizer
+    r"""a synergistic optimizer combining RAdam and LookAhead, and now GC in one optimizer.
 
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
@@ -50,18 +50,18 @@ class Ranger(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            alpha=alpha,
-            k=k,
-            step_counter=0,
-            betas=betas,
-            n_sma_threshold=n_sma_threshold,
-            eps=eps,
-            weight_decay=weight_decay,
-            adamd_debias_term=adamd_debias_term,
-            buffer=[[None, None, None] for _ in range(10)],
-        )
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'betas': betas,
+            'alpha': alpha,
+            'k': k,
+            'step_counter': 0,
+            'n_sma_threshold': n_sma_threshold,
+            'weight_decay': weight_decay,
+            'adamd_debias_term': adamd_debias_term,
+            'buffer': [[None, None, None] for _ in range(10)],
+            'eps': eps,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -72,7 +72,7 @@ class Ranger(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'Ranger'
 
     @torch.no_grad()
@@ -95,40 +95,30 @@ class Ranger(Optimizer, BaseOptimizer):
                 loss = closure()
 
         for group in self.param_groups:
+            beta1, beta2 = group['betas']
+            n_sma_max: float = 2.0 / (1.0 - beta2) - 1.0
             for p in group['params']:
                 if p.grad is None:
                     continue
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
-
-                if grad.dtype in (torch.float16, torch.bfloat16):
-                    grad = grad.float()
-
-                p_fp32 = p
-                if p.dtype in {torch.float16, torch.bfloat16}:
-                    p_fp32 = p_fp32.float()
+                    raise NoSparseGradientError(self.__str__)
 
                 state = self.state[p]
                 if len(state) == 0:
                     state['step'] = 0
-                    state['exp_avg'] = torch.zeros_like(p_fp32)
-                    state['exp_avg_sq'] = torch.zeros_like(p_fp32)
-                    state['slow_buffer'] = torch.empty_like(p_fp32)
-                    state['slow_buffer'].copy_(p_fp32)
-                else:
-                    state['exp_avg'] = state['exp_avg'].type_as(p_fp32)
-                    state['exp_avg_sq'] = state['exp_avg_sq'].type_as(p_fp32)
+                    state['exp_avg'] = torch.zeros_like(p)
+                    state['exp_avg_sq'] = torch.zeros_like(p)
+                    state['slow_buffer'] = torch.empty_like(p)
+                    state['slow_buffer'].copy_(p)
 
+                state['step'] += 1
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
-                beta1, beta2 = group['betas']
 
                 if self.use_gc and grad.dim() > self.gc_gradient_threshold:
                     grad = centralize_gradient(grad, gc_conv_only=False)
 
-                state['step'] += 1
-
                 exp_avg.mul_(beta1).add_(grad, alpha=1.0 - beta1)
                 exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1.0 - beta2)
 
@@ -140,11 +130,10 @@ class Ranger(Optimizer, BaseOptimizer):
                 else:
                     buffered[0] = state['step']
                     beta2_t = beta2 ** state['step']
-                    n_sma_max = 2 / (1 - beta2) - 1
                     n_sma = n_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
                     buffered[1] = n_sma
                     if n_sma > self.n_sma_threshold:
-                        rt = math.sqrt(
+                        step_size = math.sqrt(
                             (1 - beta2_t)
                             * (n_sma - 4)
                             / (n_sma_max - 4)
@@ -153,8 +142,6 @@ class Ranger(Optimizer, BaseOptimizer):
                             * n_sma_max
                             / (n_sma_max - 2)
                         )
-
-                        step_size = rt
                         if not group['adamd_debias_term']:
                             step_size /= bias_correction1
                     else:
@@ -162,16 +149,14 @@ class Ranger(Optimizer, BaseOptimizer):
 
                     buffered[2] = step_size
 
-                if group['weight_decay'] != 0:
-                    p_fp32.add_(p_fp32, alpha=-group['weight_decay'] * group['lr'])
+                if group['weight_decay'] > 0.0:
+                    p.add_(p, alpha=-group['weight_decay'] * group['lr'])
 
                 if n_sma > self.n_sma_threshold:
                     de_nom = exp_avg_sq.sqrt().add_(group['eps'])
-                    p_fp32.addcdiv_(exp_avg, de_nom, value=-step_size * group['lr'])
+                    p.addcdiv_(exp_avg, de_nom, value=-step_size * group['lr'])
                 else:
-                    p_fp32.add_(exp_avg, alpha=-step_size * group['lr'])
-
-                p.copy_(p_fp32)
+                    p.add_(exp_avg, alpha=-step_size * group['lr'])
 
                 if state['step'] % group['k'] == 0:
                     slow_p = state['slow_buffer']
