@@ -1,14 +1,15 @@
 import torch
 from torch.optim.optimizer import Optimizer
 
-from pytorch_optimizer.base.base_optimizer import BaseOptimizer
 from pytorch_optimizer.base.exception import NoSparseGradientError
+from pytorch_optimizer.base.optimizer import BaseOptimizer
 from pytorch_optimizer.base.types import CLOSURE, DEFAULTS, LOSS, PARAMETERS
 from pytorch_optimizer.optimizer.utils import projection
 
 
 class SGDP(Optimizer, BaseOptimizer):
-    r"""SGD + Slowing Down the Slowdown for Momentum Optimizers on Scale-invariant Weights
+    r"""SGD + Slowing Down the Slowdown for Momentum Optimizers on Scale-invariant Weights.
+
     :param params: PARAMETERS. iterable of parameters to optimize or dicts defining parameter groups.
     :param lr: float. learning rate.
     :param momentum: float. momentum factor.
@@ -40,16 +41,16 @@ class SGDP(Optimizer, BaseOptimizer):
 
         self.validate_parameters()
 
-        defaults: DEFAULTS = dict(
-            lr=lr,
-            momentum=momentum,
-            dampening=dampening,
-            weight_decay=weight_decay,
-            nesterov=nesterov,
-            eps=eps,
-            delta=delta,
-            wd_ratio=wd_ratio,
-        )
+        defaults: DEFAULTS = {
+            'lr': lr,
+            'weight_decay': weight_decay,
+            'momentum': momentum,
+            'dampening': dampening,
+            'delta': delta,
+            'wd_ratio': wd_ratio,
+            'nesterov': nesterov,
+            'eps': eps,
+        }
         super().__init__(params, defaults)
 
     def validate_parameters(self):
@@ -59,7 +60,7 @@ class SGDP(Optimizer, BaseOptimizer):
         self.validate_epsilon(self.eps)
 
     @property
-    def __name__(self) -> str:
+    def __str__(self) -> str:
         return 'SGDP'
 
     @torch.no_grad()
@@ -79,14 +80,13 @@ class SGDP(Optimizer, BaseOptimizer):
 
         for group in self.param_groups:
             momentum = group['momentum']
-
             for p in group['params']:
                 if p.grad is None:
                     continue
 
                 grad = p.grad
                 if grad.is_sparse:
-                    raise NoSparseGradientError(self.__name__)
+                    raise NoSparseGradientError(self.__str__)
 
                 state = self.state[p]
                 if len(state) == 0:
@@ -95,10 +95,9 @@ class SGDP(Optimizer, BaseOptimizer):
                 buf = state['momentum']
                 buf.mul_(momentum).add_(grad, alpha=1.0 - group['dampening'])
 
+                d_p = buf
                 if group['nesterov']:
-                    d_p = grad + momentum * buf
-                else:
-                    d_p = buf
+                    d_p.mul_(momentum).add_(grad)
 
                 wd_ratio: float = 1.0
                 if len(p.shape) > 1:
@@ -111,7 +110,7 @@ class SGDP(Optimizer, BaseOptimizer):
                         group['eps'],
                     )
 
-                if group['weight_decay'] > 0:
+                if group['weight_decay'] > 0.0:
                     p.mul_(1.0 - group['lr'] * group['weight_decay'] * wd_ratio / (1.0 - momentum))
 
                 p.add_(d_p, alpha=-group['lr'])
